@@ -1,27 +1,32 @@
 /*
- *  $Id: msgbox.c,v 1.38 2004/11/18 21:57:00 tom Exp $
+ *  $Id: msgbox.c,v 1.53 2005/12/07 00:01:04 tom Exp $
  *
  *  msgbox.c -- implements the message box and info box
  *
- *  AUTHOR: Savio Lam (lam836@cs.cuhk.hk)
- *     and: Thomas E. Dickey
+ *  Copyright 2000-2004,2005	Thomas E. Dickey
  *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License
- *  as published by the Free Software Foundation; either version 2
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as
+ *  published by the Free Software Foundation; either version 2.1 of the
+ *  License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this program; if not, write to
+ *	Free Software Foundation, Inc.
+ *	51 Franklin St., Fifth Floor
+ *	Boston, MA 02110, USA.
+ *
+ *  An earlier version of this program lists as authors:
+ *	Savio Lam (lam836@cs.cuhk.hk)
  */
 
-#include "dialog.h"
+#include <dialog.h>
+#include <dlg_keys.h>
 
 /*
  * Display the message in a scrollable window.  Actually the way it works is
@@ -54,14 +59,14 @@ show_message(WINDOW *dialog,
 	 * If we're not limited by the screensize, allow text to possibly be
 	 * one character per line.
 	 */
-	if ((len = strlen(prompt)) > high)
+	if ((len = dlg_count_columns(prompt)) > high)
 	    high = len;
 #endif
 	dummy = newwin(high, width, 0, 0);
 	wbkgdset(dummy, dialog_attr | ' ');
 	wattrset(dummy, dialog_attr);
 	werase(dummy);
-	dlg_print_autowrap(dummy, prompt, high, wide);
+	dlg_print_autowrap(dummy, prompt, high, width);
 	getyx(dummy, y, x);
 
 	copywin(dummy,		/* srcwin */
@@ -111,12 +116,45 @@ int
 dialog_msgbox(const char *title, const char *cprompt, int height, int width,
 	      int pauseopt)
 {
+    /* *INDENT-OFF* */
+    static DLG_KEYS_BINDING binding[] = {
+	ENTERKEY_BINDINGS,
+	DLG_KEYS_DATA( DLGK_ENTER,	' ' ),
+	DLG_KEYS_DATA( DLGK_GRID_DOWN,	'J' ),
+	DLG_KEYS_DATA( DLGK_GRID_DOWN,	'j' ),
+	DLG_KEYS_DATA( DLGK_GRID_DOWN,	KEY_DOWN ),
+	DLG_KEYS_DATA( DLGK_GRID_UP,	'K' ),
+	DLG_KEYS_DATA( DLGK_GRID_UP,	'k' ),
+	DLG_KEYS_DATA( DLGK_GRID_UP,	KEY_UP ),
+	DLG_KEYS_DATA( DLGK_PAGE_FIRST,	'g' ),
+	DLG_KEYS_DATA( DLGK_PAGE_FIRST,	KEY_HOME ),
+	DLG_KEYS_DATA( DLGK_PAGE_LAST,	'G' ),
+	DLG_KEYS_DATA( DLGK_PAGE_LAST,	KEY_END ),
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,	'F' ),
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,	'f' ),
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,	KEY_NPAGE ),
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,	'B' ),
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,	'b' ),
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,	KEY_PPAGE ),
+	DLG_KEYS_DATA( DLGK_FIELD_NEXT,	KEY_DOWN ),
+	DLG_KEYS_DATA( DLGK_FIELD_NEXT, KEY_RIGHT ),
+	DLG_KEYS_DATA( DLGK_FIELD_NEXT, TAB ),
+	DLG_KEYS_DATA( DLGK_FIELD_PREV,	KEY_UP ),
+	DLG_KEYS_DATA( DLGK_FIELD_PREV, KEY_BTAB ),
+	DLG_KEYS_DATA( DLGK_FIELD_PREV, KEY_LEFT ),
+	END_KEYS_BINDING
+    };
+    /* *INDENT-ON* */
+
     int x, y, last = 0, page;
+    int button = 0;
     int key = 0, fkey;
+    int result = DLG_EXIT_UNKNOWN;
     WINDOW *dialog = 0;
     char *prompt = dlg_strclone(cprompt);
     const char **buttons = dlg_ok_label();
     int offset = 0;
+    int check;
     bool show = TRUE;
 
 #ifdef KEY_RESIZE
@@ -142,7 +180,11 @@ dialog_msgbox(const char *title, const char *cprompt, int height, int width,
 	(void) refresh();
     } else
 #endif
+    {
 	dialog = dlg_new_window(height, width, y, x);
+	dlg_register_window(dialog, "msgbox", binding);
+	dlg_register_buttons(dialog, "msgbox", buttons);
+    }
     page = height - (1 + 3 * MARGIN);
 
     dlg_mouse_setbase(x, y);
@@ -153,13 +195,11 @@ dialog_msgbox(const char *title, const char *cprompt, int height, int width,
     wattrset(dialog, dialog_attr);
 
     if (pauseopt) {
-	bool done = FALSE;
-
 	dlg_draw_bottom_box(dialog);
 	mouse_mkbutton(height - 2, width / 2 - 4, 6, '\n');
-	dlg_draw_buttons(dialog, height - 2, 0, buttons, FALSE, FALSE, width);
+	dlg_draw_buttons(dialog, height - 2, 0, buttons, button, FALSE, width);
 
-	while (!done) {
+	while (result == DLG_EXIT_UNKNOWN) {
 	    if (show) {
 		getyx(dialog, y, x);
 		last = show_message(dialog, prompt, offset, page, width, pauseopt);
@@ -168,48 +208,14 @@ dialog_msgbox(const char *title, const char *cprompt, int height, int width,
 		show = FALSE;
 	    }
 	    key = dlg_mouse_wgetch(dialog, &fkey);
-	    if (!fkey) {
-		fkey = TRUE;
-		switch (key) {
-		case ESC:
-		    done = TRUE;
-		    continue;
-		case ' ':
-		case '\n':
-		case '\r':
-		    key = KEY_ENTER;
-		    break;
-		case 'F':
-		case 'f':
-		    key = KEY_NPAGE;
-		    break;
-		case 'B':
-		case 'b':
-		    key = KEY_PPAGE;
-		    break;
-		case 'g':
-		    key = KEY_HOME;
-		    break;
-		case 'G':
-		    key = KEY_END;
-		    break;
-		case 'K':
-		case 'k':
-		    key = KEY_UP;
-		    break;
-		case 'J':
-		case 'j':
-		    key = KEY_DOWN;
-		    break;
-		default:
-		    if (dlg_char_to_button(key, buttons) == 0) {
-			key = KEY_ENTER;
-		    } else {
-			fkey = FALSE;
-		    }
-		    break;
-		}
+	    if (dlg_result_key(key, fkey, &result))
+		break;
+
+	    if (!fkey && (check = dlg_char_to_button(key, buttons)) >= 0) {
+		result = check ? DLG_EXIT_HELP : DLG_EXIT_OK;
+		break;
 	    }
+
 	    if (fkey) {
 		switch (key) {
 #ifdef KEY_RESIZE
@@ -219,34 +225,52 @@ dialog_msgbox(const char *title, const char *cprompt, int height, int width,
 		    width = req_wide;
 		    goto restart;
 #endif
-		case KEY_ENTER:
-		    done = TRUE;
+		case DLGK_FIELD_NEXT:
+		    button = dlg_next_button(buttons, button);
+		    if (button < 0)
+			button = 0;
+		    dlg_draw_buttons(dialog,
+				     height - 2, 0,
+				     buttons, button,
+				     FALSE, width);
 		    break;
-		case KEY_HOME:
+		case DLGK_FIELD_PREV:
+		    button = dlg_prev_button(buttons, button);
+		    if (button < 0)
+			button = 0;
+		    dlg_draw_buttons(dialog,
+				     height - 2, 0,
+				     buttons, button,
+				     FALSE, width);
+		    break;
+		case DLGK_ENTER:
+		    result = button ? DLG_EXIT_HELP : DLG_EXIT_OK;
+		    break;
+		case DLGK_PAGE_FIRST:
 		    if (offset > 0) {
 			offset = 0;
 			show = TRUE;
 		    }
 		    break;
-		case KEY_END:
+		case DLGK_PAGE_LAST:
 		    if (offset < last) {
 			offset = last;
 			show = TRUE;
 		    }
 		    break;
-		case KEY_UP:
+		case DLGK_GRID_UP:
 		    if (offset > 0) {
 			--offset;
 			show = TRUE;
 		    }
 		    break;
-		case KEY_DOWN:
+		case DLGK_GRID_DOWN:
 		    if (offset < last) {
 			++offset;
 			show = TRUE;
 		    }
 		    break;
-		case KEY_PPAGE:
+		case DLGK_PAGE_PREV:
 		    if (offset > 0) {
 			offset -= page;
 			if (offset < 0)
@@ -254,7 +278,7 @@ dialog_msgbox(const char *title, const char *cprompt, int height, int width,
 			show = TRUE;
 		    }
 		    break;
-		case KEY_NPAGE:
+		case DLGK_PAGE_NEXT:
 		    if (offset < last) {
 			offset += page;
 			if (offset > last)
@@ -262,11 +286,14 @@ dialog_msgbox(const char *title, const char *cprompt, int height, int width,
 			show = TRUE;
 		    }
 		    break;
+		case DLGK_MOUSE(0):
+		    result = DLG_EXIT_OK;
+		    break;
+		case DLGK_MOUSE(1):
+		    result = DLG_EXIT_HELP;
+		    break;
 		default:
-		    if (key >= M_EVENT)
-			done = TRUE;
-		    else
-			beep();
+		    beep();
 		    break;
 		}
 	    } else {
@@ -275,12 +302,12 @@ dialog_msgbox(const char *title, const char *cprompt, int height, int width,
 	}
     } else {
 	show_message(dialog, prompt, offset, page, width, pauseopt);
-	key = '\n';
 	wrefresh(dialog);
+	result = DLG_EXIT_OK;
     }
 
     dlg_del_window(dialog);
     dlg_mouse_free_regions();
     free(prompt);
-    return (key == ESC) ? DLG_EXIT_ESC : DLG_EXIT_OK;
+    return result;
 }
